# -*- coding: utf-8 -*-

import datetime, random, re

def generate_random_key():
    """Generate random 30 digit (15 byte) hex string.

    stackoverflow question 2782229
    """
    return '%030x' % (random.randrange(256**15),)

def get_note_category(note):
    if 'category' in note:
        category = note['category'] if note['category'] is not None else ''
    else:
        category = ''
    return category

# Returns a fixed length string:
#   'X' - needs sync
#   '*' - favorite
def get_note_flags(note):
    flags = ''
    flags += 'X' if float(note['modified']) > float(note['syncdate']) else ' '
    if 'favorite' in note:
        flags += '*' if note['favorite'] else ' '
    else:
        flags += '  '
    return flags

def get_note_title(note):
    if 'title' in note:
        return note['title']
    else:
        return ''

def note_favorite(n):
    if 'favorite' in n:
        return n['favorite']
    else:
        return False

def sort_by_title_favorite(a):
    return (not note_favorite(a.note), get_note_title(a.note))

def sort_notes_by_categories(notes, favorite_ontop=False):
    notes.sort(key=lambda i: (favorite_ontop and not note_favorite(i.note),
                              i.note.get('category'),
                              get_note_title(i.note)))

def sort_by_modify_date_favorite(a):
    if note_favorite(a.note):
        return 100.0 * float(a.note.get('modified', 0))
    else:
        return float(a.note.get('modified', 0))

class KeyValueObject:
    """Store key=value pairs in this object and retrieve with o.key.

    You should also be able to do MiscObject(**your_dict) for the same effect.
    """

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)


def build_regex_search(search_string):
    """
    Build up a compiled regular expression from the search string.

    Supports the use of flags - ie. search for `nothing/i` will perform a
    case-insensitive regex for `nothing`
    """

    sspat = None
    valid_flags = {
            'i': re.IGNORECASE
    }
    if search_string:
        try:
            search_string, flag_letters = re.match(r'^(.+?)(?:/([a-z]+))?$', search_string).groups()
            flags = 0
            # if flags are given, OR together all the valid flags
            # see https://docs.python.org/3/library/re.html#re.compile
            if flag_letters:
                for letter in flag_letters:
                    if letter in valid_flags:
                        flags = flags | valid_flags[letter]
            sspat = re.compile(search_string, flags)
        except re.error:
            sspat = None

    return sspat
